#-----------------------------------------------------------------------------#
# STEP 45: Look for synteny with aegypti                                      #
#-----------------------------------------------------------------------------#

# Look for synteny with Ae. aegypti in an attempt to identify chromosomes.
# This takes into account the reorientation of the chromosomes.

# It uses Peter's orthologs. This is after we rejiggered chromosome three, but
# before we moved the Nix locus.

#- INITIALIZE ----------------------------------------------------------------#

setwd('D:/bsa/')
library(dplyr)
library(data.table)
library(ggplot2)

# Where the annotations are:
loc_aeg = "annotations/ae_aegypti.gff3"
loc_albo = "annotations/chr_scaffold_PR.gff3"

# Where the reoriented annotation information is:
loc_reorient = "contig_info/position_translator.txt"

# Where the list of chromosome lengths are:
loc_csize_al = "contig_info/al_chromosome_lengths.txt"
loc_csize_ae = "contig_info/ae_chromosome_lengths.txt"

# Where the infomation about the scaffolds is:
loc_chr1 = "contig_info/chr_contigs1.agp"
loc_chr2 = "contig_info/chr_contigs2.agp"
loc_chr3 = "contig_info/chr_contigs3.agp"

# Where the list of busco genes are:
loc_busco = "annotations/links_to_ODB10.txt"

# Where info on the FISH markers are:
loc_fish = "annotations/probe_set_PR_scaffolds.gff3"
loc_fish_2 = "annotations/probe_set_2_positions.txt"

# Where Peter's orthologs are:
loc_ortho = "annotations/alb_aeg_ortholog_coords.csv"

# Where to print figure:
loc_fig = "figures"

#- READ IN ANNOTATIONS -------------------------------------------------------#

ae = fread(loc_aeg, fill=TRUE, sep="\t")
al = fread(loc_albo, fill=TRUE, sep="\t")
colnames(ae) = c("chromosome", "V2", "type", "start", "end", 
                 "V6", "V7", "V8", "info")
colnames(al) = colnames(ae)
ae = ae[grepl("NC", ae$chromosome),]

orthos = fread(loc_ortho)

#- ADD GENOME POSITIONS FOR Ae. aegypti --------------------------------------#

ae_csize = fread(loc_csize_ae)
ae_chr_1_length = ae_csize$length[1==ae_csize$true_chromosome]
ae_chr_2_length = ae_csize$length[2==ae_csize$true_chromosome]
ae_chr_3_length = ae_csize$length[3==ae_csize$true_chromosome]

orthos$aeg_genome_pos = orthos$aeg_start_coord

orthos$aeg_genome_pos[orthos$aeg_chr == "chr2"] = 
  orthos$aeg_genome_pos[orthos$aeg_chr == "chr2"] + ae_chr_1_length

orthos$aeg_genome_pos[orthos$aeg_chr == "chr3"] = 
  orthos$aeg_genome_pos[orthos$aeg_chr == "chr3"] + 
    ae_chr_1_length + ae_chr_2_length

#- ADD GENOME POSITIONS FOR Ae. albopictus -----------------------------------#

t = fread(loc_reorient)
t$true_scaffold_name = paste0("chr", t$true_chromosome, ".", t$true_scaffold)

base_pos = t$true_genome_position[match(orthos$alb_scaffold, 
                                        t$true_scaffold_name)]
#rev = t$scaffold_reversed[match(al$chromosome, t$pseudo_scaffold_name)]
#s_lengths = t$scaffold_length[match(al$chromosome, t$pseudo_scaffold_name)]
add_it = orthos$alb_start_coord
#add_it[1==rev] = 1 + s_lengths[1==rev] - al$end[1==rev]

orthos$alb_genome_pos = base_pos + add_it

#- GET CHROMOSOME LENGTHS ----------------------------------------------------#

al_csize = fread(loc_csize_al)
c1_length = al_csize$length[1==al_csize$true_chromosome]
c2_length = al_csize$length[2==al_csize$true_chromosome]
c3_length = al_csize$length[3==al_csize$true_chromosome]


#- PLOT THE FISH MARKERS -----------------------------------------------------#

f = fread(loc_fish)
colnames(f) = c("pseudo_scaffold", "V2", "V3", "pseudo_scaffold_pos", "V5",
                "V6", "V7", "V8", "info")
f$band = sub("albo_(.*);.*", "\\1", f$info)
f$chr_ae = as.numeric(sub(".*aeg_(.)[pq].*", "\\1", f$info))
f$chr_pos_ae = as.numeric(sub(".*:(.*)", "\\1", f$info))
f$gen_pos_ae = f$chr_pos_ae
f$gen_pos_ae[2 == f$chr_ae] = f$gen_pos_ae[2 == f$chr_ae] + ae_csize$length[1]
f$gen_pos_ae[3 == f$chr_ae] = f$gen_pos_ae[3 == f$chr_ae] + 
                              ae_csize$length[1] + ae_csize$length[2]

f$gen_pos_al = f$pseudo_scaffold_pos + 
               t$true_genome_position[match(f$pseudo_scaffold, 
                                            t$pseudo_scaffold_name)]
f2 = fread(loc_fish_2)
f2$gen_pos_al = f2$pseudo_coordinate +
                t$true_genome_position[match(f2$pseudo_scaffold, 
                                             t$pseudo_scaffold_name)]
f2$gen_pos_ae = f2$chr_pos_ae
f2$gen_pos_ae[2 == f2$chr_ae] = f2$chr_pos_ae[2 == f2$chr_ae] + ae_chr_1_length
f2$gen_pos_ae[3 == f2$chr_ae] = f2$chr_pos_ae[3 == f2$chr_ae] + 
                                ae_chr_1_length + ae_chr_2_length

fa = rbind(f, f2, fill=T)

pdf(file=paste0(loc_fig, "/figureD_raw_PetersOrthos.pdf"), width=11, height=8)
print(
ggplot() + 
  geom_point(data=orthos,
             aes(alb_genome_pos, aeg_genome_pos), 
             alpha=0.05, pch=16, size = 3) +
  xlab("Position in Ae. albopictus genome") + 
  ylab("Position in Ae. aegypti genome") +
  geom_vline(xintercept = 0) + 
  geom_vline(xintercept = c1_length) + 
  geom_vline(xintercept = c1_length + c2_length + c3_length) + 
  geom_vline(xintercept = c1_length + c2_length) +
  geom_hline(yintercept = 0) +
  geom_hline(yintercept = ae_chr_1_length) +
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length + ae_chr_3_length)+
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length) +
  ggtitle(paste0("FISH probes overlaid on syntenic genes")) +
  geom_label(data=fa, aes(gen_pos_al, gen_pos_ae, label=band)) +
  geom_point(data=fa, aes(gen_pos_al, gen_pos_ae), pch=16, size=4, color="red")+
  theme_classic()
)
dev.off()

#- FIN -----------------------------------------------------------------------#