#-----------------------------------------------------------------------------#
# STEP 45: Look for synteny with aegypti                                      #
#-----------------------------------------------------------------------------#

# Look for synteny with Ae. aegypti in an attempt to identify chromosomes.

#- INITIALIZE ----------------------------------------------------------------#

setwd('C:/Users/Jack/Documents/bsa/')
library(dplyr)
library(data.table)
library(ggplot2)

# Where the annotations are:
loc_aeg = "annotations/ae_aegypti.gff3"
loc_albo = "annotations/chr_scaffold_PR.gff3"

# Where the infomation about the scaffolds is:
loc_chr1 = "contig_info/chr_contigs1.agp"
loc_chr2 = "contig_info/chr_contigs2.agp"
loc_chr3 = "contig_info/chr_contigs3.agp"

# Where the list of busco genes are:
loc_busco = "annotations/links_to_ODB10.txt"

#- READ IN ANNOTATIONS -------------------------------------------------------#

ae = fread(loc_aeg, fill=TRUE, sep="\t")
al = fread(loc_albo, fill=TRUE, sep="\t")
colnames(ae) = c("chromosome", "V2", "type", "start", "end", 
                 "V6", "V7", "V8", "info")
colnames(al) = colnames(ae)
ae = ae[grepl("NC", ae$chromosome),]

#- MATCH GENE NAMES TO GENE IDS ----------------------------------------------#

aem = ae %>% filter(type == "mRNA")
alm = al %>% filter(type == "mRNA")

get_names = function(v) {
  r = sub(".*product=(.*?);.*", "\\1", v, perl=T)
  r = sub("%2C transcript variant.*", "", r)
  return(r)
}

aem$name = get_names(aem$info)
alm$name = get_names(alm$info)
aem$id = sub(".*Dbxref=GeneID:(.*?),.*", "\\1", aem$info, perl=T)
alm$id = sub(".*Dbxref=GeneID:(.*?),.*", "\\1", alm$info, perl=T)

ae_id_to_name = aem %>% group_by(id) %>% summarize(name = unique(name))
al_id_to_name = alm %>% group_by(id) %>% summarize(name = unique(name))

#- FILTER DOWN TO GENES ------------------------------------------------------#

ae = ae %>% filter(type == "gene")
al = al %>% filter(type == "gene")
ae$id = sub(".*Dbxref=GeneID:(.*?);.*", "\\1", ae$info, perl=T)
al$id = sub(".*Dbxref=GeneID:(.*?);.*", "\\1", al$info, perl=T)

#- GET NAMES -----------------------------------------------------------------#

ae$name = ae_id_to_name$name[match(ae$id, ae_id_to_name$id)]
al$name = al_id_to_name$name[match(al$id, al_id_to_name$id)]

#- FILTER OUT UNIDENTIFIED STUFF AND FIND CROSS-OVER -------------------------#

ae = ae %>% filter(!is.na(name))
al = al %>% filter(!is.na(name))

ae = ae[!grepl("uncharacterized LOC", ae$name),]
al = al[!grepl("uncharacterized LOC", al$name),]

ae = ae %>% filter(name %in% al$name)
al = al %>% filter(name %in% ae$name)

#- ADD GENOME POSITIONS FOR Ae. aegypti --------------------------------------#

ae$genome_pos = ae$start
ae_chr_1_length = max(ae %>% filter(chromosome == "NC_035107.1") %>% 
                             pull(start))
ae$genome_pos[ae$chromosome == "NC_035108.1"] = 
  ae$genome_pos[ae$chromosome == "NC_035108.1"] + ae_chr_1_length

ae_chr_2_length = max(ae %>% filter(chromosome == "NC_035108.1") %>%
                             pull(start))
ae$genome_pos[ae$chromosome == "NC_035109.1"] = 
  ae$genome_pos[ae$chromosome == "NC_035109.1"] + 
  ae_chr_1_length + ae_chr_2_length

#- ADD GENOME POSITIONS FOR Ae. albopictus -----------------------------------#

cumsum_getter = function(v) {
  returnatus = rep(0, length(v))
  for (i in 2:length(v)) {
    returnatus[i] = returnatus[i-1] + v[i-1]
  }
  return(returnatus)
}

al$chr_no = sub("LG(.*)\\..*", "\\1", al$chromosome)
al$scaf_no = sub(".*\\.(.*)", "\\1", al$chromosome)
l = al %>% group_split(chr_no)

# For chromosome 1:
c1 = fread(loc_chr1)
c1$scaffold_number = as.numeric(sub(".*\\.(.*)", "\\1", c1$V1))
c1_scaffold_lengths = c1 %>% group_by(scaffold_number) %>%
  summarize(length=max(V3))
c1_scaffold_lengths$cumsum = cumsum_getter(c1_scaffold_lengths$length)
c1_length = sum(c1_scaffold_lengths$length)

l[[1]]$start_chromosome = l[[1]]$start +
  c1_scaffold_lengths$cumsum[match(l[[1]]$scaf_no, 
                                   c1_scaffold_lengths$scaffold_number)]
l[[1]]$start_genome = l[[1]]$start_chromosome

# For chromosome 2:
c2 = fread(loc_chr2)
c2$scaffold_number = as.numeric(sub(".*\\.(.*)", "\\1", c2$V1))
c2_scaffold_lengths = c2 %>% group_by(scaffold_number) %>%
  summarize(length=max(V3))
c2_scaffold_lengths$cumsum = cumsum_getter(c2_scaffold_lengths$length)
c2_length = sum(c2_scaffold_lengths$length)

l[[2]]$start_chromosome = l[[2]]$start +
  c2_scaffold_lengths$cumsum[match(l[[2]]$scaf_no, 
                                   c2_scaffold_lengths$scaffold_number)]
# Add a buffer so I can work with windows of 1 MB or less:
l[[2]]$start_genome = l[[2]]$start_chromosome +
  ceiling(c1_length / 1000000) * 1000000

# For chromosome 3:
c3 = fread(loc_chr3)
c3$scaffold_number = as.numeric(sub(".*\\.(.*)", "\\1", c3$V1))
c3_scaffold_lengths = c3 %>% group_by(scaffold_number) %>%
  summarize(length=max(V3))
c3_scaffold_lengths$cumsum = cumsum_getter(c3_scaffold_lengths$length)
c3_length = sum(c3_scaffold_lengths$length)

l[[3]]$start_chromosome = l[[3]]$start +
  c3_scaffold_lengths$cumsum[match(l[[3]]$scaf_no, 
                                   c3_scaffold_lengths$scaffold_number)]
# Add a buffer so I can work with windows of 1 MB or less:
l[[3]]$start_genome = l[[3]]$start_chromosome +
  ceiling(c1_length / 1000000) * 1000000 +
  ceiling(c2_length / 1000000) * 1000000

al = l %>% bind_rows()

#- COMBINE SETS --------------------------------------------------------------#

a = full_join(ae %>% select(name, genome_pos),
              al %>% select(name, start_genome))
ggplot() + 
  geom_point(data=a, aes(start_genome, genome_pos), 
             alpha=0.03, pch=16, size = 3) +
  xlab("Position in Ae. albopictus genome") + 
  ylab("Position in Ae. aegypti genome") +
  geom_vline(xintercept = c1_length) + 
  geom_vline(xintercept = c1_length + c2_length) +
  geom_hline(yintercept = ae_chr_1_length) +
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length) +
  ggtitle(paste0("Synteny of all genes, n=", length(unique(a$name)))) +
  theme_classic()

#- FILTER TO ONLY BUSCO GENES ------------------------------------------------#

b = fread(loc_busco, header=F)

ab = a %>% filter(name %in% b$V2)

ggplot() + 
  geom_point(data=ab, aes(start_genome, genome_pos), 
             alpha=0.15, pch=16, size = 3) +
  xlab("Position in Ae. albopictus genome") + 
  ylab("Position in Ae. aegypti genome") +
  geom_vline(xintercept = c1_length) + 
  geom_vline(xintercept = c1_length + c2_length) +
  geom_hline(yintercept = ae_chr_1_length) +
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length) +
  ggtitle(paste0("Synteny of BUSCO genes, n=", length(unique(ab$name)))) + 
  theme_classic()

#- FILTER TO ONLY SINGLE-COPY GENES ------------------------------------------#

ae_single_copy = ae %>% group_by(name) %>% 
                        summarize(copy_no = n()) %>%
                        filter(copy_no == 1) %>% 
                        pull(name)
al_single_copy = al %>% group_by(name) %>% 
                        summarize(copy_no = n()) %>%
                        filter(copy_no == 1) %>% 
                        pull(name)


ggplot() + 
  geom_point(data=a %>% filter(name %in% ae_single_copy |
                               name %in% al_single_copy),
             aes(start_genome, genome_pos), 
             alpha=0.05, pch=16, size = 3) +
  xlab("Position in Ae. albopictus genome") + 
  ylab("Position in Ae. aegypti genome") +
  geom_vline(xintercept = c1_length) + 
  geom_vline(xintercept = c1_length + c2_length) +
  geom_hline(yintercept = ae_chr_1_length) +
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length) +
  ggtitle(paste0("Synteny of genes that are single-copy in either genome, n=",
                 a %>% filter(name %in% ae_single_copy | 
                              name %in% al_single_copy) %>% pull(name) %>%
                              unique %>% length)) +
  theme_classic()

ggplot() + 
  geom_point(data=a %>% filter(name %in% ae_single_copy &
                               name %in% al_single_copy),
             aes(start_genome, genome_pos), 
             alpha=0.05, pch=16, size = 3) +
  xlab("Position in Ae. albopictus genome") + 
  ylab("Position in Ae. aegypti genome") +
  geom_vline(xintercept = c1_length) + 
  geom_vline(xintercept = c1_length + c2_length) +
  geom_hline(yintercept = ae_chr_1_length) +
  geom_hline(yintercept = ae_chr_1_length + ae_chr_2_length) +
  ggtitle(paste0("Synteny of genes that are single-copy in both genomes, n=",
                 a %>% filter(name %in% ae_single_copy & 
                              name %in% al_single_copy) %>% pull(name) %>%
                              unique %>% length)) +
  theme_classic()
